///////////////////////////////////////////////////////////
// ODLISTB2.CPP: Demonstrates an owner-draw list box with
//               both graphical and text items.  Each listbox item's
//               text has a different color
//
// Prepared by : George Cross, Borland C++ Developer Support
// Date        : June 17, 1996
// Original
// Author      : Clayton Walnum in his excellent book Borland C++
//               Tips, Tricks and Traps, Que publications (Thanks Clayton)
//
// Complier    : Borland C++ 5.0, but is valid for OWL 2.x and above.
//
///////////////////////////////////////////////////////////

#include <owl/pch.h>
#include <owl/listbox.h>
#include <owl/gdiobjec.h>
#include "odlistb2.rc"

//////////////////////////////////
// The application class.
//////////////////////////////////
class TApp : public TApplication
{
public:
   TApp(): TApplication() {}
   void InitMainWindow();
};

//////////////////////////////////
// The main window class.
//////////////////////////////////
class TWndw : public TFrameWindow
{
public:
   TWndw(TWindow *parent, const char far *title);

protected:
   void CmTestDialog();

   DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1(TWndw, TFrameWindow)
  EV_COMMAND(CM_TESTDIALOG, CmTestDialog),
END_RESPONSE_TABLE;

//////////////////////////////////
// The dialog box class.
//////////////////////////////////
class TDlg: public TDialog
{
protected:
   char *lbText[8];
   TListBox *listBox;

public:
   TDlg(TWindow *parent, TResId resId);

protected:
   void SetupWindow();
   void EvDrawItem(UINT, DRAWITEMSTRUCT far &draw);
   void EvMeasureItem(UINT, MEASUREITEMSTRUCT far &meas);
   LRESULT EvCompareItem(UINT,
      COMPAREITEMSTRUCT far &comp);

   DECLARE_RESPONSE_TABLE(TDlg);
};

DEFINE_RESPONSE_TABLE1(TDlg, TDialog)
   EV_WM_DRAWITEM,
   EV_WM_MEASUREITEM,
   EV_WM_COMPAREITEM,
END_RESPONSE_TABLE;

///////////////////////////////////////////////////////////
// TWndw::TWndw()
///////////////////////////////////////////////////////////
TWndw::TWndw(TWindow *parent, const char far * title) :
   TFrameWindow(parent, title)
{
   AssignMenu(MENU_1);
   Attr.X = 50;
   Attr.Y = 50;
   Attr.W = 400;
   Attr.H = 300;
}

///////////////////////////////////////////////////////////
// TWndw::CmTestDialog()
///////////////////////////////////////////////////////////
void TWndw::CmTestDialog()
{
   TDialog *dialog = new TDlg(this, DIALOG_1);
   dialog->Execute();
}

///////////////////////////////////////////////////////////
// TDlg::TDlg()
///////////////////////////////////////////////////////////
TDlg::TDlg(TWindow *parent, TResId resId) :
   TDialog(parent, resId)
{
   // Construct the list box.
   listBox = new TListBox(this, IDC_LISTBOX);
}

///////////////////////////////////////////////////////////
// TDlg::SetupWindow()
///////////////////////////////////////////////////////////
void TDlg::SetupWindow()
{
   TDialog::SetupWindow();

	// Initialize the text-item array.
   // Always do this before calling AddString().
   lbText[0] = "Apples";
   lbText[1] = "Balloons";
   lbText[2] = "Cats";
   lbText[3] = "Dogs";
   lbText[4] = "Echoes";
   lbText[5] = "Farms";
   lbText[6] = "Gardens";
   lbText[7] = "Hobbies";

   // Add items to the list box.
   listBox->AddString((LPSTR) ICON_1);
   listBox->AddString((LPSTR) ICON_2);
   listBox->AddString((LPSTR) ICON_3);
   listBox->AddString((LPSTR) ICON_4);
   listBox->AddString((LPSTR) ICON_5);
   listBox->AddString((LPSTR) ICON_6);
	listBox->AddString((LPSTR) ICON_7);
	listBox->AddString((LPSTR) ICON_8);
}

///////////////////////////////////////////////////////////
// TDlg::EvDrawItem()
///////////////////////////////////////////////////////////
void TDlg::EvDrawItem(UINT, DRAWITEMSTRUCT far &draw)
{
   TRect rect;
   HBRUSH hBrush;

   // Calculate the size of the focus rectangle.
   rect.left = draw.rcItem.left;
   rect.top = draw.rcItem.top;
   rect.right = rect.left + GetSystemMetrics(SM_CXICON);
   rect.bottom = rect.top + GetSystemMetrics(SM_CYICON);

   // Construct a device context.
   TDC drawDC(draw.hDC);

	// If the list-box item is gaining
	// or losing the focus...
   if (draw.itemAction & ODA_FOCUS)

      // Draw or erase the focus rectangle.
      drawDC.DrawFocusRect(rect);

   // If the item needs to be drawn...
   else if (draw.itemAction & (ODA_DRAWENTIRE | ODA_SELECT))
   {
      // Get the brush and text color for a
      // selected or unselected item.
      if (draw.itemState & ODS_SELECTED)
      {
         hBrush = (HBRUSH) GetStockObject(BLACK_BRUSH);
         drawDC.SetTextColor(TColor::LtMagenta);
      }
      else
      {
         hBrush = (HBRUSH) GetStockObject(LTGRAY_BRUSH);

         //Use a random color for the text.  PALETTEINDEX macro works nice.
         drawDC.SetTextColor(TColor(PALETTEINDEX (draw.itemID)));
      }

      // Fill the item rectangle with the background color.
      drawDC.FillRect(draw.rcItem, hBrush);

		// Load the appropriate icon.
      HINSTANCE hInstance = *GetModule();
      #pragma warn -sig
		TIcon icon(hInstance, (TResId) draw.itemData);

		// Draw the icon in the correct position
		// for this item.
		drawDC.DrawIcon(draw.rcItem.left,
			draw.rcItem.top, (HICON) icon);

		// Calculate the text index.
		int index = draw.itemData - 1;

      // Calculate the text's location.
		int x = rect.right + 8;
      int y = rect.top + 8;

      // Draw the text.
      drawDC.TextOut(x, y, lbText[index]);

      // If the item needs a focus rectangle, draw it.
      if (draw.itemState & ODS_FOCUS)
         drawDC.DrawFocusRect(rect);
   }
}

///////////////////////////////////////////////////////////
// TDlg::EvMeasureItem()
///////////////////////////////////////////////////////////
void TDlg::EvMeasureItem(UINT, MEASUREITEMSTRUCT far &meas)
{
   // Calculate the height of an item in the list box.
   meas.itemHeight = GetSystemMetrics(SM_CYICON) + 2;
}

///////////////////////////////////////////////////////////
// TDlg::EvCompareItem()
///////////////////////////////////////////////////////////
LRESULT TDlg::EvCompareItem(UINT,
   COMPAREITEMSTRUCT far &comp)
{
   // Get the strings to compare.
   char *s1 = lbText[comp.itemData1-1];
   char *s2 = lbText[comp.itemData2-1];

   // Compare the strings and return the result.
   return lstrcmp(s1, s2);
}

///////////////////////////////////////////////////////////
// TApp::InitMainWindow()
///////////////////////////////////////////////////////////
void TApp::InitMainWindow()
{
   EnableBWCC();
   TFrameWindow *wndw =
      new TWndw(0, "Owner-Draw List Box App 2");

   SetMainWindow(wndw);
}

///////////////////////////////////////////////////////////
// OwlMain()
///////////////////////////////////////////////////////////
int OwlMain(int, char*[])
{
   return TApp().Run();
}

